<?php

namespace Drupal\tmgmt_webt\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\Html;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\Data;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use Drupal\webt\translation_engines\etranslation\EtranslationService;
use Drupal\webt\TranslationHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * WEB-T translator plugin.
 *
 * @TranslatorPlugin(
 *   id = "webt",
 *   label = @Translation("WEB-T"),
 *   description = @Translation("WEB-T Translator service."),
 *   ui = "Drupal\tmgmt_webt\WebtTranslatorUi",
 *   logo = "icons/webt.svg",
 * )
 */
class WebtTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {

  use StringTranslationTrait;

  /**
   * The WEB-T service.
   *
   * @var Drupal\webt\translation_engines\etranslation\EtranslationService
   */
  protected $client;

  /**
   * TMGMT data helper.
   *
   * @var \Drupal\tmgmt\Data
   */
  protected Data $tmgmtData;

  /**
   * If the process is being run via cron or not.
   *
   * @var bool|null
   */
  protected ?bool $isCron;

  /**
   * Constructs a DeeplProTranslator object.
   *
   * @param Drupal\webt\translation_engines\etranslation\EtranslationService $client
   *   The WEB-T translation service.
   * @param \Drupal\tmgmt\Data $tmgmt_data
   *   The TMGMT data object.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(EtranslationService $client, Data $tmgmt_data, array $configuration, string $plugin_id, array $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->tmgmtData = $tmgmt_data;
    $this->isCron = NULL;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    // @phpstan-ignore-next-line
    return new static(
      $container->get('webt.translation_service.etranslation'),
      $container->get('tmgmt.data'),
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function requestTranslation(JobInterface $job): void {
    $this->requestJobItemsTranslation($job->getItems());
    if (!$job->isRejected()) {
      $job->submitted('The translation job has been submitted.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultRemoteLanguagesMappings(): array {
    return [
      'ar' => 'AR',
      'bg' => 'BG',
      'cs' => 'CS',
      'da' => 'DA',
      'de' => 'DE',
      'el' => 'EL',
      'en' => 'EN',
      'es' => 'ES',
      'et' => 'ET',
      'fi' => 'FI',
      'fr' => 'FR',
      'ga' => 'GA',
      'hr' => 'HR',
      'hu' => 'HU',
      'is' => 'IS',
      'it' => 'IT',
      'ja' => 'JA',
      'lt' => 'LT',
      'lv' => 'LV',
      'nb' => 'NB',
      'nl' => 'NL',
      'pl' => 'PL',
      'pt' => 'PT',
      'ro' => 'RO',
      'ru' => 'RU',
      'sk' => 'SK',
      'sl' => 'SL',
      'sv' => 'SV',
      'tr' => 'TR',
      'uk' => 'UK',
      'zh' => 'ZH',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedRemoteLanguages(TranslatorInterface $translator): array {
    // Pre-defined array of available languages.
    return [
      'AR' => $this->t('Arabic'),
      'BG' => $this->t('Bulgarian'),
      'CS' => $this->t('Czech'),
      'DA' => $this->t('Danish'),
      'DE' => $this->t('German'),
      'EL' => $this->t('Greek'),
      'EN' => $this->t('English'),
      'ES' => $this->t('Spanish'),
      'ET' => $this->t('Estonian'),
      'FI' => $this->t('Finnish'),
      'FR' => $this->t('French'),
      'GA' => $this->t('Irish'),
      'HR' => $this->t('Croation'),
      'HU' => $this->t('Hungarian'),
      'IS' => $this->t('Icelandic'),
      'IT' => $this->t('Italian'),
      'JA' => $this->t('Japanese'),
      'LT' => $this->t('Lithuanian'),
      'LV' => $this->t('Latvian'),
      'NB' => $this->t('Norwegian (Bokmål)'),
      'NL' => $this->t('Dutch'),
      'PL' => $this->t('Polish'),
      'PT' => $this->t('Portuguese'),
      'RO' => $this->t('Romanian'),
      'RU' => $this->t('Russian'),
      'SK' => $this->t('Slovak'),
      'SL' => $this->t('Slovenian'),
      'SV' => $this->t('Swedish'),
      'TR' => $this->t('Turkish'),
      'UK' => $this->t('Ukrainian'),
      'ZH' => $this->t('Chinese'),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @phpstan-ignore-next-line
   */
  public function getSupportedTargetLanguages(TranslatorInterface $translator, $source_language): array {
    $languages = $this->getSupportedRemoteLanguages($translator);

    // There are no language pairs, any supported language can be translated
    // into the others. If the source language is part of the languages,
    // then return them all, just remove the source language.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items): void {
    $job_item = reset($job_items);
    if ($job_item instanceof JobItemInterface) {
      /** @var \Drupal\tmgmt\Entity\Job $job */
      $job = $job_item->getJob();
      foreach ($job_items as $job_item) {
        if ($job->isContinuous()) {
          $job_item->active();
        }
        // Pull the source data array through the job and flatten it.
        $data = $this->tmgmtData->filterTranslatable($job_item->getData());

        $text = [];
        $keys_sequence = [];
        $translation = [];
        $i = 0;

        // Traverse array and get text parts.
        foreach ($data as $key => $value) {
          $escaped_text = $this->escapeText($value);
          $text[] = $escaped_text;
          $keys_sequence[] = $key;
        }

        $source = $job->getRemoteSourceLanguage();
        $target = $job->getRemoteTargetLanguage();

        try {

          // Exctract only the parts to translate.
          $strings_to_translate = TranslationHelper::extractTranslatableHtmlValues($text);

          // Request for translation.
          $translation_response = $this->client->translate($source, $target, $strings_to_translate);

          // Build the translated values.
          $translations = TranslationHelper::mergeTranslatedHtmlValues($text, $strings_to_translate, $translation_response->translations, TRUE);

          // Build response array.
          foreach ($translations as $translated) {
            $translation[$keys_sequence[$i]]['#text'] = Html::decodeEntities($translated);
            $i++;
          }

          $job_item->addTranslatedData($this->tmgmtData
            ->unflatten($translation));
        }
        catch (TMGMTException $e) {
          $job->rejected('Translation has been rejected with following error: @error',
            ['@error' => $e->getMessage()], 'error');
        }

      }
    }
  }

}
